#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2014-10-06 17:19:06
# @Author  : Han Zhao (han.zhao@uwaterloo.ca)
# @Link    : https://github.com/KeiraZhao
# @Version : $Id$
import os, sys
import unittest
import theano
import theano.tensor as T
import numpy as np
import scipy as sp
import logging
import time
import csv
import cPickle
import traceback
import random
import argparse
import matplotlib
import matplotlib.pyplot as plt

from scipy.sparse import *

sys.path.append('../source/')

from grcnn import AdaSent
from config import AdaSentConfiger
from wordvec import WordEmbedding
from utils import floatX

parser = argparse.ArgumentParser()
parser.add_argument('-m', '--model', help='Using model trained before',
                    type=str, default=None)
parser.add_argument('name', action='store', type=str)
# Compile and configure all the parameters
args = parser.parse_args()
# Set meta-parameters of the model
# Logging configuration
# Set the basic configuration of the logging system
log_formatter = logging.Formatter(fmt='%(asctime)s [%(processName)s, %(process)s] [%(levelname)-5.5s]  %(message)s', 
                                  datefmt='%m-%d %H:%M')
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
# Stdout logger
std_handler = logging.StreamHandler(sys.stdout)
std_handler.setFormatter(log_formatter)
std_handler.setLevel(logging.DEBUG)
logger.addHandler(std_handler)
# Different processing for different data set
start_time = time.time()
logger.debug('Name of data set: {}'.format(args.name))
data_name = args.name.split('-')[0]
if data_name == 'MR':
    mr_positive_filename = '../data/movie-review/mr-polarity.pos'
    mr_negative_filename = '../data/movie-review/mr-polarity.neg'
    # Loading and building training and test data set
    data_txt, data_label = [], []
    # Read all the instances 
    with file(mr_positive_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Positive Instances: %d' % len(lines))
        data_txt.extend(lines)
        data_label.extend([1] * len(lines))
    with file(mr_negative_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Negative Instances: %d' % len(lines))
        data_txt.extend(lines)
        data_label.extend([0] * len(lines))
    # Shuffling all the data
    assert len(data_txt) == len(data_label)
    data_size = len(data_txt)
    logger.debug('Size of the data sets: %d' % data_size)
    random_index = np.arange(data_size)
    np.random.shuffle(random_index)
    data_txt = list(np.asarray(data_txt)[random_index])
    data_label = list(np.asarray(data_label)[random_index])
elif data_name == 'CR':
    cr_filename = '../data/custom-review/customer-review.txt'
    # Loading and building training and test data set
    data_txt, data_label = [], []
    # Read all the instances
    with file(cr_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Number of training instances in the data set: %d' % len(lines))
        for line in lines:
            segments = line.split(',')
            data_txt.append(segments[1])
            data_label.append(int(segments[0]))
    # Shuffling all the data
    assert len(data_txt) == len(data_label)
    data_size = len(data_txt)
    random_index = np.arange(data_size)
    np.random.shuffle(random_index)
    data_txt = list(np.asarray(data_txt)[random_index])
    data_label = list(np.asarray(data_label)[random_index])
elif data_name == 'SUBJ':
    subj_positive_filename = '../data/subjectivity/quote.tok.gt9.5000'
    subj_negative_filename = '../data/subjectivity/plot.tok.gt9.5000'
    # Loading and building training and test data set
    data_txt, data_label = [], []
    # Read all the instances 
    with file(subj_positive_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Positive Instances: %d' % len(lines))
        data_txt.extend(lines)
        data_label.extend([1] * len(lines))
    with file(subj_negative_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Negative Instances: %d' % len(lines))
        data_txt.extend(lines)
        data_label.extend([0] * len(lines))
    # Shuffling all the data
    assert len(data_txt) == len(data_label)
    data_size = len(data_txt)
    logger.debug('Size of the data sets: %d' % data_size)
    random_index = np.arange(data_size)
    np.random.shuffle(random_index)
    data_txt = list(np.asarray(data_txt)[random_index])
    data_label = list(np.asarray(data_label)[random_index])
elif data_name == 'MPQA':
    mpqa_filename = '../data/MPQA/mpqa.txt'
    # Loading and building training and test data set
    data_txt, data_label = [], []
    # Read all the instances
    with file(mpqa_filename, 'r') as fin:
        lines = fin.readlines()
        logger.debug('Number of training instances in the data set: %d' % len(lines))
        for line in lines:
            segments = line.split(',')
            data_txt.append(segments[1])
            data_label.append(int(segments[0]))
    # Shuffling all the data
    assert len(data_txt) == len(data_label)
    data_size = len(data_txt)
    random_index = np.arange(data_size)
    np.random.shuffle(random_index)
    data_txt = list(np.asarray(data_txt)[random_index])
    data_label = list(np.asarray(data_label)[random_index])
elif data_name == 'TREC':
    trec_train_filename = '../data/TREC/train_5500.label.txt'
    trec_test_filename = '../data/TREC/TREC_10.label.txt'
    # Label mapping
    label_map = {'DESC':0, 'ENTY':1, 'ABBR':2, 'HUM':3, 'LOC':4, 'NUM':5}
    # Loading and building training and test data set
    data_txt, data_label = [], []
    # Read all the instances 
    with file(trec_train_filename, 'r') as fin:
        for line in fin:
            tokens = line.split()
            data_txt.append(' '.join(tokens[1:]))
            data_label.append(tokens[0].split(':')[0])
    with file(trec_test_filename, 'r') as fin:
        for line in fin:
            tokens = line.split()
            data_txt.append(' '.join(tokens[1:]))
            data_label.append(tokens[0].split(':')[0])
    # Shuffling all the data
    assert len(data_txt) == len(data_label)
    data_size = len(data_txt)
    logger.debug('Size of the data set: %d' % data_size)
    # Mapping string into int
    data_label = map(lambda x: label_map[x], data_label)
    # Random shuffling for training and test data set
    index = np.arange(data_size)
    np.random.shuffle(index)
    data_txt = list(np.asarray(data_txt)[index])
    data_label = list(np.asarray(data_label)[index])
else:
    raise Exception('No data set named: {}'.format(args.name))
    sys.exit(-1)
end_time = time.time()
logger.debug('Data set load finished, time used for loading and shuffling: {} seconds.'.format(end_time-start_time))
# Typological drawing
logger.debug('Loading model: {}'.format(args.model))
start_time = time.time()
adasent = AdaSent.load('{}-adasent-highest.model'.format(args.name))
word_embedding = WordEmbedding.load('{}-word2vec-highest.model'.format(args.name))
embed_dim = word_embedding.embedding_dim()
end_time = time.time()
logger.debug('Time used to load the models: {} seconds.'.format(end_time-start_time))
# Mapping
while True:
    idx = input('Please enter the index of the sentence to be presented: ')
    sentence = data_txt[idx]
    label = data_label[idx]
    # Show the decision of experts and their relative weights
    words = sentence.split()
    words = [word.lower() for word in words]
    vectors = np.zeros((len(words)+2, embed_dim), dtype=np.float32)
    vectors[1:-1] = np.asarray([word_embedding.wordvec(word) for word in words])
    # Plotting
    weights = adasent.show_weights(vectors)
    experts = adasent.show_scores(vectors)[:,1][np.newaxis, :]
    image = np.asarray([weights[0], experts[0]])
    logger.debug('=' * 50)
    logger.debug('Input sentence: {}'.format(sentence))
    logger.debug('Weights of hierarchy: {}'.format(weights))
    logger.debug('Classification of hierarchy: {}'.format(experts))
    logger.debug('True label of the instance: {}'.format(label))
    logger.debug('Predicted label = {}'.format(np.sum(weights * experts)))
    ticks = np.linspace(0.0, 1.0, 11, endpoint=True)
    plt.figure()
    plt.imshow(image, cmap=matplotlib.cm.Blues, interpolation='nearest')
    plt.colorbar(ticks=ticks, orientation='horizontal')
    plt.clim(0.0, 1.0)
    plt.xticks([])
    plt.yticks([])
    plt.title(r'True label = {}, $\Pr(y=1|\mathbf{{x}})$ = {:1.3f}'.format(label, np.sum(weights * experts)))
    plt.tight_layout()
    plt.savefig('{}-{}.png'.format(data_name, idx), bbox_inches='tight')
    plt.show()
